package Freenet;
import Freenet.message.*;
import Freenet.support.*;
import java.util.*;


public class StandardHandshakeHandler implements HandshakeHandler {

    private Hashtable handshakes;
    private volatile Hashtable waiters;

    private class Waiter {
	public Address addr;
	public Waiter(Address addr) {
	    this.addr = addr;
	}
    }

    public StandardHandshakeHandler() {
	handshakes = new Hashtable();
	waiters = new Hashtable();
    }

    protected class HandshakeWaiter implements Callback {

	private Thread me;

	public HandshakeWaiter(Thread me) {
	    this.me = me;
	}

	public void callback() {
	    me.interrupt();
	}	
    }
    /**
     * Sends a handshake over a given connection. Please note that the
     * handshake will register as belonging to the port which this
     * connection is connected to, if you did not initiate the connection
     * this is probably not the same port the node listens on - and you should
     * use the method that connects itself instead.
     * @param ch   A ConnectionHandler serving the connection to send over
     * @returns    true if a handshake from this node has been served in
     *             the last Core.handshakeLife milliseconds, or if it replied
     *             to a HandshakeRequest within Core.handshakeTimeout
     *             milliseconds.  Else false.
     **/

    public boolean getHandshake(Core n, ConnectionHandler ch)
    {
	Date d=(Date)handshakes.get(ch.peer());
	if(d!=null) {
	    if((d.getTime()+Core.handshakeLife) < new Date().getTime()) {
		handshakes.remove(ch.peer());
		d=null;
	    } else
		return true;
	}

	HandshakeRequest hr=new HandshakeRequest(new Random().nextLong());
	Waiter w = new Waiter(ch.peer());
	try {
	    waiters.put(new Long(hr.id),w);
	    hr.sending(n, ch);
	    ch.sendMessage(hr);
	} catch(SendFailedException e) {
	    Logger.log("HandshakeHandler.java","Couldn't send handshake request",Logger.MINOR); 
	    return false;
	}

	try {
	    synchronized(w) {
		w.wait(Core.handshakeTimeout);
	    }
	} catch(InterruptedException e) {
	}

	d=(Date)handshakes.get(w.addr);
	Logger.log("HandShakeHandler.java","Done waiting for handshake, returning " + (d != null),Logger.DEBUG);

	return d != null;
    }



    public boolean getHandshake(Core n, Address addr)
    {
	Date d=(Date)handshakes.get(addr);
	if(d!=null) {
	    if((d.getTime()+Core.handshakeLife) < new Date().getTime()) {
		handshakes.remove(addr);
		d=null;
	    } else
		return true;
	}
	try {
	    ConnectionHandler ch = n.connect(addr);
	    return getHandshake(n,ch);
	} catch (ConnectFailedException c) {
	    return false;
	}
    }

    public void setHandshake(long id) {	

	try {
	    Waiter w = (Waiter) waiters.get(new Long(id));
	    handshakes.put(w.addr, new Date()); 
	    synchronized(w) {
		w.notifyAll();
	    }
	} catch (ClassCastException e) {
	} catch (NullPointerException e) {
	}

    }
}
