package Freenet.contrib.FGUI;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import Freenet.support.ByteCounter;

/**
 * A progress bar that happens to be a ByteCounter.
 */
public class ProgressBar implements ByteCounter {
    final static Color COLOR = new Color(20, 100, 150);


    Component owner;
    boolean done;
    boolean visible;
    long length;
    long bytes;
    String label;
    
    
    public ProgressBar(Component owner,  String label) {
	this.owner = owner;
	this.visible = true;
    this.done = false;
    }

    void setVisible(boolean visible) {
	if (visible != this.visible) {
	    this.visible = visible;
	    repaint();
	    
	}
    }


    public void start(long length, String label) {
    	this.label = label;
		this.length = length;
		bytes = 0;
		System.out.println("start" + label);
    }
    
    public void count(int n) {
	bytes += n;
	repaint();
    }
    
    public void finished() {
	setVisible(false);
		done = true;
    }
	
	void repaint() {
		owner.repaint();
    }

	public void paint(Graphics g, Rectangle rect) {
		if (!visible) {
			    return;
			}
		
		g.setColor(Color.black);
		double percent = (double)bytes / (double)length;

		if (label != null)
	 		{g.drawString(label, rect.x, rect.y - 5);}
		if (percent >= 0) {
		    int pw = (int)(((double)rect.width) * percent);
		
		    
		    g.setColor(COLOR);
		    g.fillRect(rect.x, rect.y, pw, rect.height);
		    g.setColor(Color.lightGray);
		    g.fillRect(rect.x + pw, rect.y, rect.width - pw, rect.height);
		    g.setColor(Color.black);
		    g.drawRect(rect.x, rect.y, rect.width, rect.height);
		  
		    	
		    g.drawString(sizeName(bytes) + " / " + sizeName(length), rect.x, rect.y + rect.height + 15);
		} else {
		    g.setColor(Color.lightGray);
		    g.fillRect(rect.x, rect.y, rect.width, rect.height);
		    g.setColor(Color.black);
		    g.drawRect(rect.x, rect.y, rect.width, rect.height);
		}
    }

	/**
	 * Convert size in bytes to human readable string.
	 */
	String sizeName(long in)
	{
		if (in > 1024*1024) {
			String str =  Double.toString((double)in / (1024.0 * 1024.0));
			return (str+"00000").substring(0,5) + " MBytes";
		}
		else if (in > 1024) {
			return (in / 1024) + " KBytes";
		}
			return in + " Bytes";
	}
	
}
