package Freenet.contrib.fproxy;
import Freenet.*;
import Freenet.client.*;
import Freenet.crypt.*;
import Freenet.message.*;
import Freenet.support.*;
import java.io.*;
import java.math.*;
import java.util.*;

/*
  This code is part of fproxy, an HTTP proxy server for Freenet.
  It is distributed under the GNU Public Licence (GPL) version 2.  See
  http://www.gnu.org/ for further details of the GPL.
*/


/**
 * Wrapper to send Freenet requests on behalf of the HttpHandler
 *
 * @author <a href="http://www.doc.ic.ac.uk/~twh1/">Theodore Hong</a>
 **/

public class ProxyClient
{
    // local variables
    public RequestClient rc;

    private ListeningAddress lstaddr;
    private String serverAddress;
    private long htl;
    private OutputStream out;


    /**
     * Creates a new ProxyClient with the given parameters
     **/
    public ProxyClient (int listenPort, String serverAddress, long htl,
			OutputStream out) {
	Logger.log("ProxyClient.java", "In constructor", Logger.DEBUGGING);
	this.lstaddr = new ListeningAddress("tcp/" + listenPort);
	this.serverAddress = serverAddress;
	this.htl = htl;
	this.out = out;
    }

    /**
     * Main routine to wrap a Freenet request
     **/
    public void request (String keyString) {
	// hash the key
	SHA1 sha = new SHA1(true);
	StringKey key = new StringKey(sha.doHash(keyString));

	// set parameters
	Core.connectTimeout = 30000;
        Core.handshakeTimeout = 30000;
	Core.handshakeLife=1000000;
        Core.connectTimeout=10000;
        Core.timer=new Ticker(500);

	// create Freenet client
	Logger.log("ProxyClient.java", "Creating request client",
		   Logger.DEBUGGING);
	rc = new RequestClient(new Address(serverAddress), lstaddr, htl,
			       (Math.abs((new Random()).nextLong()) % 30));
	rc.setCloseOnExit(false);

	PrintWriter pw = new PrintWriter(out);
	DataReply reply;
	InputStream stream;
	
	// do it
	try {
	    reply = rc.request(key);
            stream = reply.in;
        } catch (Exception e) {

	    // something went wrong - send error message to browser
	    // for now, HttpHandler will send the header
	    // pw.print("Content-type: text/plain\015\012\015\012");
	    pw.print("\n");
	    pw.println("Couldn't retrieve key: "+keyString);

	    if (e instanceof BadHandshakeException) {
		pw.println("Bad handshake from node");
	    } else if (e instanceof SendFailedException) {
		pw.print("Couldn't connect to the specified Freenet node - ");
		if (serverAddress.startsWith("tcp/localhost")) {
		    pw.println("have you started it?");
		}
		else {
		    pw.println("please restart fproxy with a different one.");
		}
	    } else if (e instanceof ClientException) {
		ClientException ce = (ClientException) e;
		if (ce.msg == null) {
		    pw.println("Network timed out waiting for a reply from the Freenet node.");
		} else if (ce.msg instanceof RequestFailed) {
		    pw.println("The key you requested could not be found anywhere in the network."); 
		} else if (ce.msg instanceof TimedOut) {
		    pw.println("The key you requested could not be found within HTL hops.");
		} else {
		    pw.println("Received unexpected reply: " + ce.msg);
		}
	    }
	    pw.flush();
	    return;
	}

	// start feeding data through
	// for now, HttpHandler will send the header
	// pw.print("Content-length: " + reply.length.longValue() + "\015\012");
	// pw.print("Content-type: text/plain\015\012\015\012");
	// pw.flush();
	Logger.log("ProxyClient.java",
		   "\n--------------------------- data follows -----------------------------",
		   Logger.NORMAL);
	Conduit cond = new Conduit(stream, out, null);
	cond.asyncFeed(null, null, reply.length.longValue());
    }
}
