package Freenet.message;
import java.io.*;
import Freenet.*;
import Freenet.node.*;
import Freenet.support.*;

/*
  This code is part of the Java Adaptive Network Client by Ian Clarke. 
  It is distributed under the GNU Public Licence (GPL) version 2.  See
  http://www.gnu.org/ for further details of the GPL.

  Explanation of Code Versions: 
    0.0.0      = Initial Description
    0.0.1      = API Specified
    0.x (x>0)  = Partial Implementation
    x.0 (x>0)  = Operational
		
  Requires Classes: Node (1.0)
                    Address (1.0)
		    Message (1.0)
 */

/**
 * This is the DataReply message
 *
 * @see Node
 * @see Address
 * @author Brandon Wiley (blanu@uts.cc.utexas.edu)
 * @author Ian Clarke (I.Clarke@strs.co.uk)
 **/

public class DataReply extends DataSend
{

    public static final String messageName = "DataReply";

  public DataReply(long idnum, long htl, long depth, Address dsrc)
    {
      super(idnum, htl, depth, dsrc);
      keepAlive = false;
    }

    public DataReply(long idnum, long htl, long depth, Address dsrc, Data data) throws IOException, DataNotReadyException{

	super(idnum, htl, depth, dsrc, data);
	keepAlive = false;
    }

  public DataReply(RawMessage raw) throws InvalidMessageException
  {
    super(raw);
    keepAlive = false;
  }

  public RawMessage toRawMessage()
  {
    RawMessage raw=super.toRawMessage();
    raw.messageType="DataReply";
    return raw;
  }

    public void sending(Core n, Address peer, Address myAddress) throws SendFailedException
    {
	if (myAddress.equals(peer))
	    throw new SendFailedException(peer); // don't send to yourself
	source = myAddress;

	// Set dataSource to my address if it is unknown, this message was never received, this message was received from a different network, or a random probability.
	if ( dataSource == null || receivedAt == null || !receivedAt.equalsHost(myAddress) || ((new java.util.Random()).nextInt() % 30) == 0)
	    {
		this.dataSource = myAddress;
	    }
    }

      // Protected/Private Methods

    protected MessageMemory timeOut(Node n, MessageMemory sb)
    {
	if (sb == null || sb.depth <= 0) return null;

	TimedOut to=new TimedOut(id, sb.depth);
    
	try {
	    to.sendBack(n,sb);
	} catch(SendFailedException sfe) {
	    Logger.log("DataReply.java","Send failed on timedout",Logger.NORMAL);
	}

        return super.timeOut(n,sb);
    }
}


