package Freenet.message;

import Freenet.*;
import Freenet.node.*;
import java.util.*;
import Freenet.support.Logger;
/*
  This code is part of the Java Adaptive Network Client by Ian Clarke. 
  It is distributed under the GNU Public Licence (GPL) version 2.  See
  http://www.gnu.org/ for further details of the GPL.
 */

/**
 * This is the superclass of all handshakes
 *
 * @author Brandon Wiley (blanu@uts.cc.utexas.edu)
 **/

public class HandshakeReply extends Handshake
{

    public static final String messageName = "HandshakeReply";

  public String version;
    public String build;
    public String revision;

  public HandshakeReply(long idnum)
    {
      super(idnum);
      version=Node.protocolVersion;
      build=Node.buildNumber;
      revision=Node.serverRevision;
    }

  public HandshakeReply(RawMessage raw) throws InvalidMessageException
  {
    super(raw);
    version=popField("Version");
    popField("Build");
    popField("Revision");
  }

  public RawMessage toRawMessage()
    {
      RawMessage r = super.toRawMessage();
      r.messageType="HandshakeReply";
      r.setField("Version", version);
      if (build != null)
	  r.setField("Build", build);
      if (revision != null)
	  r.setField("Revision",revision);
      r.trailingFieldName="EndMessage";
      return r;
    }

  public boolean check()
  {
    return version.equals(Node.protocolVersion);
  }

  public MessageMemory pReceived(Node n, MessageMemory sb)
  {
      if(check()) {
	  Logger.log("HandshakeReply.java","Setting handshake from " + source,Logger.MINOR);
	  n.hh.setHandshake(id);
      } else
	  Logger.log("HandshakeReply.java","Bad handshake received",Logger.MINOR);
      return null;
  }

    protected MessageMemory timeOut(Node n, MessageMemory sb)
    {
	return pReceived(n, sb);
    }
}


