/*
  This code is part of the Java Adaptive Network Client by Ian Clarke. 
  It is distributed under the GNU Public Licence (GPL) version 2.  See
  http://www.gnu.org/ for further details of the GPL.
 */

/**
 * This is the HandshakeRequest message
 *
 * @author Brandon Wiley (blanu@uts.cc.utexas.edu)
 **/

package Freenet.message;
import Freenet.*;
import Freenet.node.*;
import java.util.*;
import Freenet.support.*;

public class HandshakeRequest extends Handshake
{

    public static final String messageName = "HandshakeRequest";

    private boolean keepalive;

    public HandshakeRequest(long idnum) {
	super(idnum);
    }


  public HandshakeRequest(RawMessage raw) throws InvalidMessageException {
      super(raw);
  } 

  public MessageMemory pReceived(Node n, MessageMemory sb)
  {
        HandshakeReply shake = new HandshakeReply(id);
	
	try {
	    ConnectionHandler ch;
	    if (receivedWith == null) {
		ch = n.connect(source);
		shake.keepAlive = false;
	    } else if (!receivedWith.isOpen()) {
	    	ch = n.connect(source);
	    } else
		ch = receivedWith;
	    shake.sending(n,ch);
	    ch.sendMessage(shake);

	} catch(SendFailedException e) {Logger.log("HandshakeRequest.java","Could not send handshake reply",Logger.MINOR);
	} catch(ConnectFailedException e) {Logger.log("HandshakeRequest.java","Could not send handshake reply",Logger.MINOR);
	}
	return null;
  }

  public RawMessage toRawMessage()
    {
      RawMessage r = super.toRawMessage();
      r.messageType="HandshakeRequest";
      return r;
    }

    protected MessageMemory timeOut(Node n, MessageMemory sb) {
	return pReceived(n, sb);
    }

}
