
package Freenet.message;

import java.io.*;
import Freenet.*;
import Freenet.node.*;
import Freenet.support.*;

public class InsertRequest extends Request {

    public static final String messageName = "InsertRequest";

  public InsertRequest(long idnum, long htl, long depth, Key key)
  {
      super(idnum, htl, depth, key);
  }

  public InsertRequest(RawMessage raw) throws InvalidMessageException
  {
      super(raw);
  }

  public RawMessage toRawMessage()
  {
    RawMessage raw=super.toRawMessage();
    raw.messageType="InsertRequest";
    return raw;
  }

    public static KeyedMM failedTimedOut(Node n, long id, KeyedMM kmm) {
	// Cancel existing timeout
	n.timer.cancel(id);

	InsertReply ir = new InsertReply(id, kmm.depth);
	try {
	    ir.sendBack(n,kmm);
	} catch(SendFailedException sfe) {
	    Logger.log("InsertReply.java","Send of InsertReply message failed",Logger.NORMAL);
	}

	kmm.dataref=null; // messages stop here
	return kmm;
    }

    protected void refFound(Address ref, Node n) throws Request.RequestAbortException {
	Logger.log("message/InsertRequest","Key collision, received InsertRequest for previously known key, requesting its original data",Logger.MINOR);
	DataRequest dr = new DataRequest(id,hopsToLive,depth,searchKey);
	dr.initSources(receivedAt,source,receivedWith);
	throw new Request.RequestAbortException(dr.pReceived(n,null)); // Handle this as request and return result
    }

    protected void dataFound(Data data, Node n) throws Request.RequestAbortException {
	// do the same as ref (I could just return the data right here, but since situations like read failing are handled better by the DataRequest code anyways
	refFound(receivedAt, n);
    }

    protected MessageMemory timeOut(Node n, MessageMemory mm) {
	Data data;
	Address ref;

	if ((data=n.ds.searchData(searchKey))!=null) { // I have the data
	    try {
		dataFound(data, n);
	    } catch (Request.RequestAbortException rae) {
		return rae.mm;
	    }
	} else if ((ref=n.ds.searchRef(searchKey))!=null) { // Reference found
	    try {
		refFound(ref, n);
	    } catch (Request.RequestAbortException rae) {
		return rae.mm;
	    }
	}

	InsertReply ir = new InsertReply(id, depth);
	try {
	    sendReply(n, ir);
	} catch(SendFailedException sfe) {
	    Logger.log("Message.java","Send of InsertReply message failed",Logger.NORMAL);
	}

	if (mm == null || !(mm instanceof KeyedMM)) {
	    KeyedMM kmm;
	    kmm = new KeyedMM(source,depth,null,searchKey,null,this.getClass());
	    kmm.pending = false;
	    return kmm;
	} else {
	    ((KeyedMM) mm).pending = false; 
	    // if the search looped here then the DataInsert will have set the reference to null already when it gets here the second time, so we don't need to (and shouldn't, so we don't interfere with the first passage) do anything.
	    return mm;
	}
    }
}
