package Freenet.message;
import Freenet.*;
import Freenet.support.Callback;
import java.util.*;
/*
  This code is part of the Java Adaptive Network Client by Ian Clarke. 
  It is distributed under the GNU General Public Licence (GPL) 
  version 2.  See http://www.gnu.org/ for further details of the GPL.
 */

/**
 * This is the MessageMemory pertaining to messages that need to
 * remember the keys used.
 *
 * @author <A HREF="mailto:I.Clarke@strs.co.uk">Ian Clarke</A>
 **/

public class KeyedMM extends MessageMemory
{
    // Public Fields
    Key searchKey, lastAttempt;  
    Class messageType;
    Vector usedAddresses;
    Address lastAddr;

    // this is the address that data is sent to, originally it is set back to the sender, but can be reset for Inserts for example
    Address dataref;


    // pending gives whether a search is still pending for this chain. It is not necessary to set this currently, just a performance issue (it won't try to cancel the timer when this MM dies if this is false).
    boolean pending = true;
  
  // Constructor
  /**
   * @param origRec The address from which the DataRequest
   *                originated
   * @param searchKey The key being searched for
   * @param lastAttempt The key corresponding to the most
   *                    recent attempt to send a DataRequest
   *                    (used as a mask).
   **/
  public KeyedMM(Address origRec,
		   long depth,
		   Key searchKey,
		   Key lastAttempt,
		 Class messageType)
    {
      super(origRec,depth);
      this.dataref = origRec;
      this.searchKey = searchKey;
      this.lastAttempt = lastAttempt;
      this.messageType = messageType;
      this.usedAddresses = new Vector();
    }

  public KeyedMM(Address origRec,
		   long depth,
		   ConnectionHandler replyCon,
		   Key searchKey,
		   Key lastAttempt,
		 Class messageType)
    {
      super(origRec,depth, replyCon);
      this.dataref = origRec;
      this.searchKey = searchKey;
      this.lastAttempt = lastAttempt;
      this.messageType = messageType;
      this.usedAddresses = new Vector();
    }


    // cancel the timer, and if it exists (meaning some node is probably waiting for response, use the RequestCBs method to sendback a requestfailed.

    public void lost(Long longid) {
	if (pending) {
	    Callback cb = Core.timer.cancel(longid.longValue());
	    if (cb != null && cb instanceof RequestCB) {
		((RequestCB) cb).sendback();
	    }
	}
    }

}
