package Freenet.message;
import Freenet.*;
import Freenet.node.*;
import Freenet.support.*;

public class RequestCB implements Callback
{
    protected Node n;
    protected long id;
    protected MessageMemory mm;
    protected long hopsToLive;

    public RequestCB(Node n, MessageMemory mm, long id, long hopsToLive)
    {
	this.n = n;
	this.mm = mm;
	this.id = id;
	this.hopsToLive = hopsToLive;
    }

    public void callback()
    {
	(new CallbackThread()).start();
    }

    private class CallbackThread extends Thread {
    
	public void run()
	{
	    // note that the if the send fails, the query won't be restarted, since if it can't send the notification, it probably won't be able to send the reply
	    try {
		Logger.log("message/RequestCB.java","Restarting message " + Long.toHexString(id) + " on no reply.",Logger.DEBUGGING);
		KeyedMM kmm = null;
		ConnectionHandler ch;
		if( mm instanceof KeyedMM )
		    kmm = (KeyedMM)mm;
		if (kmm != null && kmm.replyCon != null && kmm.replyCon.isOpen()) {
		    ch = kmm.replyCon;
		} else {
		    ch = n.makeConnection(mm.origRec);
		}
		QueryRestarted qrm = new QueryRestarted(id, mm.depth);
		qrm.sending(n, ch);
		ch.sendMessage(qrm, null, null);
		n.timer.add(id, hopsToLive * Node.timePerHop, RequestCB.this); // HACK!, Failed needs to cancel a timer to reschedule it
		RequestFailed rf = new RequestFailed(id, hopsToLive);
		n.mh.handle(rf); // handle as if a failed message were received
	    } catch (SendFailedException sfe) {
		Logger.log("message/RequestCB.java","Couldn't restart on timeout because send failed to " + sfe.peer, Logger.NORMAL);
	    } catch (ConnectFailedException sfe) {
		Logger.log("message/RequestCB.java","Couldn't restart on timeout because connect failed to " + sfe.peer, Logger.NORMAL);
	    }
	}
    }

    // this differs from the callback in that it sends back a requestfailed, rather then attemption to restart here. Used when pending RequestCBs are canceled because they were pushed from the MessageHandlers hashtable
    public void sendback() {
	(new SendbackThread()).start();
    }

    private class SendbackThread extends Thread {
	public void run() {
	    try {
		Logger.log("message/RequestCB.java","Failing to serve request do to overflowing MessageHandler",Logger.NORMAL);
		RequestFailed rf = new RequestFailed(id, hopsToLive);
		rf.sendBack(n, mm);
	    } catch (SendFailedException sfe) {
		Logger.log("message/RequestCB.java","Failed to fail on sendback",Logger.DEBUGGING);
	    }
	}
    }
}
