package Freenet.node;
import Freenet.Address;
import Freenet.Key;
import Freenet.support.Callback;
import java.io.Serializable;
import java.io.IOException;

public interface DataStore extends Serializable, Callback {

    public void tofile(String filename) throws IOException;

  /**
   * Searches for a key in the DataStore and if found returns
   * any data associated with that key.  If none is found,
   * null is returned.
   * @param k The key to search for
   * @return The data associated with the key or null if not found
   **/
    public Data searchData(Key k);
  /**
    * Searches for a key in the DataStore and if found returns
    * any reference associated with that key.  If none is found,
    * null is returned.
    * @param k The key to search for
    * @return The reference associated with the key or null
    *         if not found
    **/
    public Address searchRef(Key k);
  /**
      * Removes an item from the DataStore given its key
    * @param k The key of the item to remove
    **/
    public void remove(Key k);
  /**
    * Returns the reference associated with the closest key
    * to k.
    * @param k The key to search for
    * @return The closest Key to k
    **/
    public Key findClosestKey(Key k);
  /**
    * Returns the reference associated with the closest key
    * to k.  The masking key allows the 'next best' key to
    * be found.  Imagine the entries in the datastore being
    * sorted in terms of closeness to the key you specify.
    * With no mask key the reference associated with the
    * top-most item will be returned.  However, with a mask
    * key any references associated with keys above the mask
    * key (including the mask key itself) will be ignored.
    * This facility allows best-first backtracking.
    * @param k The key to search for
    * @param maskKey The masking key (or null for no mask)
    * @return The closest Key to k (excluding masked keys) or
    *         null if all keys are masked.
    **/
    public Key findClosestKey(Key k, Key maskKey);

  /**
    * Adds an item to the DataStore
    * @param k The key to add
    * @param r The address to which requests should be directed for
    *          this key if the data is removed
    * @param d The data associated with this key
    **/
    public void put(Key k, Address r, Data d);

}
