package Freenet.node;
import Freenet.*;
import Freenet.crypt.*;
import Freenet.support.*;
import Freenet.message.HandshakeRequest;
import java.util.*;
import java.net.*;
import java.io.*;

/*
  This code is part of the Java Adaptive Network Client by Ian Clarke. 
  It is distributed under the GNU Public Licence (GPL) version 2.  See
  http://www.gnu.org/ for further details of the GPL.
*/

/**
 * This is a Wrapper object that contains the components of a Node in the
 * Adaptive Network.  It uses a Network object to communicate with other
 * Nodes in the Network.
 *
 * @author <A HREF="mailto:I.Clarke@strs.co.uk">Ian Clarke</A>
 * @author <a href="mailto:blanu@uts.cc.utexas.edu">Brandon Wiley</a>
 **/

public class Node extends Core
{
    public static Node n;
    static final String BUILD = "LATEST_BUILD="; // Look in inform.php

    public static void main(String[] args)
    {
      Hashtable at=new Hashtable();
      // set revision number for handshake
      StringTokenizer cvsId = new StringTokenizer ("$Id: Node.java,v 1.15 2000/04/08 20:48:17 hobbex Exp $", " ");
      cvsId.nextToken();
      cvsId.nextToken();
      serverRevision = cvsId.nextToken();

      try {
        Data.path = new File(".freenet");
        if (!Data.path.exists())
	    Data.path.mkdir();

	// parse command line
	String[] rcfiles = {".freenetrc", "freenet.ini"};
	Logger.threshold = Logger.ERROR;
        params = new Params(rcfiles, args);

	// check for help
	if (params.getParam("help") != null) {
	    usage();
	    return;
	}
	else if (params.getParam("version") != null) {
	    version();
	    return;
	}

        if (params.getParam("logging")==null)
            Logger.threshold = Logger.NORMAL;        
	else if (params.getParam("logging").equalsIgnoreCase("error"))
	    Logger.threshold = Logger.ERROR;
	else if (params.getParam("logging").equalsIgnoreCase("normal"))
	    Logger.threshold = Logger.NORMAL;
	else if (params.getParam("logging").equalsIgnoreCase("minor"))
	    Logger.threshold = Logger.MINOR;
	else if (params.getParam("logging").equalsIgnoreCase("debugging"))
	    Logger.threshold = Logger.DEBUGGING;
        Logger.verbosity = params.getint("verbosity",Logger.verbosity);

	String fname = params.getParam("logFile","NO");
	if (!fname.equalsIgnoreCase("NO")) {
	    try {
		Logger.logto(fname);
	    } catch (Exception e) {
		System.out.println("Writing to log failed");
	    }
	}
	
        // timer must be initialized before datastore so that datastore
	// can register an event to write it to disk periodically
        Node.timer = new Ticker(params.getlong("tickerTime",500));

	// give the DataStore a filename to write that reflects the port
	StandardDataStore.filename = ".freenet/store_" + params.getint("listenPort",0);

        // set parameters
        // These should be taken from arguments or a config file
        n = new Node(params.getlong("diskCache", 500), 
		          params.getint("dataStoreSize", 500), 
			  params.getint("messageStoreSize", 500), 
			  new ListeningAddress
			      ("tcp/" + params.getint("listenPort", 0)));
	timePerHop = params.getint("timePerHop",5000);
        tunneling = !((params.getParam("Tunnel", "YES")).equalsIgnoreCase("NO"));
        connectTimeout = params.getint("connectTimeout",30000);
        handshakeTimeout = params.getint("handshakeTimeout",30000);
	handshakeLife = params.getint("handshakeLife",10000000);
        if (!tunneling)
	    Logger.log("Node.java","Tunneling turned off",Logger.DEBUGGING);

        try
        {
	 if ( (params.getParam("informRead").equalsIgnoreCase("yes")
            || params.getParam("informWrite").equalsIgnoreCase("yes"))
            && params.getParam("informUrl") != null)
          inform(at);
        }
        catch(NullPointerException e)
        {
          Logger.log("Node.java","informRead/Write not set",Logger.ERROR);
        }

        try
            {
                BufferedReader br=new BufferedReader(new FileReader("nodes.config"));
                StringKey tname;
                SHA1 sha=new SHA1(true);
                while(br.ready())
                    {
                        String addrstr=br.readLine();
			addrstr=addrstr.trim();
			if(!addrstr.equals(""))
                        {
                          tname = new StringKey(sha.doHash(addrstr));
                          if(n.ds.searchRef(tname)==null)
                            at.put(addrstr, tname);
                        }
                    }
            }
        catch(IOException e) {}
        
	Enumeration iterator=at.keys();
	while(iterator.hasMoreElements())
	{
	  String addr=(String)iterator.nextElement();
	  try {
	    n.ds.put((StringKey)at.get(addr), new Address(addr), null);
	  } catch(Exception e) {e.printStackTrace();}
	}

        n.acceptConnections();
      }
      catch(Exception e) {e.printStackTrace();}
    }

    private static final void informWrite(HttpURLConnection c) throws IOException
    {
	if (params.getParam("informWrite").equalsIgnoreCase("yes"))
	{
	  c.setRequestProperty("Content-type", "application/x-www-form-urlencoded");
	  PrintWriter outStream = new PrintWriter(c.getOutputStream());
	  outStream.print("port="+params.getParam("listenPort"));
	  outStream.close();
	}
    }

    private static final void informRead(HttpURLConnection c, Hashtable at) throws IOException
    {
	if (params.getParam("informRead").equalsIgnoreCase("yes"))
	{
	  InputStreamReader ir = new InputStreamReader(c.getInputStream());
	  BufferedReader br = new BufferedReader(ir);
          SHA1 sha=new SHA1(true);
	  while(br.ready())
	  {
	    String addrstr=br.readLine();
	    addrstr=addrstr.trim();
	    if(addrstr.startsWith(BUILD))
            {
	      String latestbuild = addrstr.substring(BUILD.length());
	      if (buildNumber.compareTo(latestbuild) < 0)
	        System.err.println ( "Newer build "+latestbuild+" is available than this build "+buildNumber+", please upgrade!");
	    }
	    else if(!addrstr.equals(""))
	    {
	      StringKey tname = new StringKey(sha.doHash(addrstr));
	      if(n.ds.searchRef(tname)==null)
	      at.put(addrstr, tname);
	    }
	  }
	  ir.close();
        }
    }

    private static final void inform(Hashtable at)
    {
      try
      {
	HttpURLConnection c = (HttpURLConnection)(new URL(params.getParam("informUrl"))).openConnection();
	c.setDoInput(true);
	c.setDoOutput(true);
	c.setUseCaches(true);
	
        informWrite(c);
        informRead(c, at);
        c.disconnect();
      }
      catch (Exception e)
      {
          Logger.log("Node.java","Inform Connection failed:"+e, Logger.ERROR);
      }
    }
    
    /**
     * Print version information
     **/
    public static void version() {
	System.out.println("Freenet server version " + freenetVersion + " (build " + buildNumber + " revision " + serverRevision + ")");
    }


    /**
     * Print usage information
     **/
    public static void usage() {
	version();

	System.out.println("Usage: freenet_server [options]");
	System.out.println("");
	System.out.println("  -connectTimeout msec       Time to wait for connection to remote node");
	System.out.println("  -dataStoreSize n           Max number of items to keep in store");
	System.out.println("  -diskCache bytes           Max size of disk cache");
	System.out.println("  -handshakeLife msec        Time to treat old handshakes as good");
	System.out.println("  -handshakeTimeout msec     Time to wait for handshake from remote node");
	System.out.println("  -informRead yes|no         Read initial nodes from list on webserver");
	System.out.println("  -informWrite yes|no        Add this node to list on webserver");
	System.out.println("  -informUrl url             URL of node list");
	System.out.println("  -listenPort port           Local port to listen on");
	System.out.println("  -logFile file              Name of log file (`no' to log to console)");
	System.out.println("  -logging error|normal|minor|debugging");
	System.out.println("                             Logging level");
	System.out.println("  -messageStoreSize n        Max number of pending messages to track");
	System.out.println("  -timePerHop msec           Time to wait for replies (per hop)");
	System.out.println("  -tunnel yes|no             Start forwarding messages while incomplete");
	System.out.println("  -verbosity 1-5             Verbosity of log messages");
	System.out.println("  -help                      Display this help and exit");
	System.out.println("  -version                   Output version information and exit");
	System.out.println("");
	System.out.println("Send bug reports to freenet-dev@lists.sourceforge.net");
    }


    public DataStore ds;

    public Node(long dataLen, int dataStoreSz, int messStoreSz, ListeningAddress lstaddr)
    {

        super(lstaddr, null, new StandardHandshakeHandler());
	mh = new StandardMessageHandler(this, messStoreSz);

	ds = StandardDataStore.makeDataStore(this, dataLen, dataStoreSz);
    }


}
