package Freenet.node;
import Freenet.*;
import java.util.*;
import Freenet.support.*;
/*
  This code is part of the Java Adaptive Network Client by Ian Clarke. 
  It is distributed under the GNU Public Licence (GPL) version 2.  See
  http://www.gnu.org/ for further details of the GPL.

  Explanation of Code Versions: 
    0.0.0      = Initial Description
    0.0.1      = API Specified
    0.x (x>0)  = Partial Implementation
    x.0 (x>0)  = Operational
		
 */

/**
 * This class handles incoming messages.
 *
 * @version $Revision: 1.3 $
 * @author <A HREF="mailto:I.Clarke@strs.co.uk">Ian Clarke</A>
 **/

public class StandardMessageHandler implements MessageHandler
{
  // Public Fields
  // Protected/Private Fields

  public Node node;
  private LimitedHashtable messageMemories;
  private LimitedHashtable tickets;
  
  public static void main(String[] args)
    {
      System.out.println("Testing limitedHashtable");
      Hashtable t = new LimitedHashtable(3);
      t.put(new Integer(1), new String("one"));
      System.out.println(t.toString());
      t.put(new Integer(2), new String("two"));
      System.out.println(t.toString());
      t.put(new Integer(3), new String("three"));
      System.out.println(t.toString());
      t.put(new Integer(4), new String("four"));
      System.out.println(t.toString());
      t.put(new Integer(5), new String("five"));
      System.out.println(t.toString());
      t.put(new Integer(6), new String("six"));
      System.out.println(t.toString());
    }

  // Constructors

  /**
   * @param n The node that should be used to send messages
   *          and for which the datastore should be updated
   * @param m The number of message objects that will be
   *          remembered
   **/
  public StandardMessageHandler(Node n, int m)
    {
      node = n;
      messageMemories = new LimitedHashtable(m);
      tickets = new LimitedHashtable(m);
    }

  // Public Methods

  /**
   * Handle a message
   * @param message The message to handle
   **/
  public void handle(Message message)
    {
      try {
	Long longid = new Long(message.id);
	Object ticket = getTicket(longid);
	synchronized(ticket) {
	  MessageMemory mm = (MessageMemory)messageMemories.get(longid);
	  MessageMemory newmm = message.received(node, mm);
	  if (newmm != mm) {
	    messageMemories.remove(longid);
	    if (newmm != null) {
	      Object[] lmm = messageMemories.limput(longid, newmm);
	      if (lmm != null) { // dispose of lost mm
		  ((MessageMemory) lmm[1]).lost((Long) lmm[0]);
	      }
	    }
	  }
	}
      } catch(Exception e) {e.printStackTrace();}
    }

  // Protected/Private Methods

  // Return the canonical object corresponding to the messageid
  // Create one if it does not already exist.  Synchronize to
  // protect against race conditions.
  synchronized Object getTicket(Long longid)
  {
    Object ticket = tickets.get(longid);
    if (ticket == null) {
      ticket = new Object();
      tickets.put(longid, ticket);
    }
    return ticket;
  }

}


/**
 * A variant on a Hashtable which only holds a limited number of
 * messages.  When more messages than that are put in, old ones are
 * deleted, oldest first.
 *
 * @version Rrevision$
 * @author <A HREF="mailto:I.Clarke@strs.co.uk">Ian Clarke</A>
 **/
class LimitedHashtable extends Hashtable
{
  Object[] ar;
  int ptr = 0;
  public LimitedHashtable(int size)
    {
      super();
      ar = new Object[size];
    }

    /**
     * @return  If an old object was pushed out of Hashtable by this
     *          insert, it's an array containing it's key (0) and data (1) 
     *           is returned. Otherwise null. Observe that 
     *          any object previously under this key will _not_ be
     *          returned (use normal put for that).
     **/
  public synchronized Object[] limput(Object key, Object value)
    {
      Object[] old = null;
      if (ar[ptr] != null)
	{
	    Object oldkey = ar[ptr];
	    Object oldvalue = remove(ar[ptr]);
	    if (oldvalue != null) {
		old = new Object[2];
		old[0] = oldkey;
		old[1] = oldvalue;
	    }
	}
      ar[ptr] = key;
      ptr++;
      if (ptr == ar.length)
	{
	  ptr = 0;
	}
      super.put(key,value);
      return old;
    }

  public synchronized Object put(Object key, Object value)
    {
      if (ar[ptr] != null)
	{
	  remove(ar[ptr]);
	}
      ar[ptr] = key;
      ptr++;
      if (ptr == ar.length)
	{
	  ptr = 0;
	}
      return super.put(key,value);
    }

  public String toString()
    {
      return super.toString() + ", max: "+ar.length;
    }
}
