#!/bin/bash
# makeDist.sh - build binary Freenet distributions

# locations
TMP=/tmp
RELATIVE_DISTDIR=freenet
DISTDIR=$TMP/$RELATIVE_DISTDIR
TARFILE=freenet.tgz
ZIPFILE=freenet.zip

# check that Freenet was built already
echo -n "Checking for compiled Node.class... "
if test ! -e ../node/Node.class; then
    echo "no"
    echo "Please run the ./build.sh script first."
    echo ""
    exit
fi
echo "yes"

# check for old distribution directory
if test -e $DISTDIR; then
    echo "Removing old temporary directory $DISTDIR"
    rm -rf $DISTDIR
fi
mkdir $DISTDIR

# make jar
echo "Creating jar file $DISTDIR/Freenet.jar"
cd ../..
jar -cvmf Freenet/node.manifest $DISTDIR/freenet.jar Freenet/*.class Freenet/{client,crypt,message,node,support}/*.class
chmod 644 $DISTDIR/freenet.jar

# add rest of files
cd Freenet
echo "Adding scripts"
install -m 0644 README $DISTDIR/README
install -m 0644 COPYING $DISTDIR/COPYING
cd scripts
install -m 0644 sample.freenetrc $DISTDIR/.freenetrc
install -m 0755 freenet_server.jar.in $DISTDIR/freenet_server
install -m 0755 freenet_insert.jar.in $DISTDIR/freenet_insert
ln -f $DISTDIR/freenet_insert $DISTDIR/freenet_request

# make tar file
echo "Creating tar archive $TARFILE"
tar -C $TMP -cvzf $TARFILE freenet

# add DOS files
echo "Setting up DOS distribution"
rm -f $DISTDIR/freenet_{server,insert,request}
rm -f $DISTDIR/.freenetrc
install -m 0644 freenet.ini $DISTDIR/freenet.ini
install -m 0755 fserve.jar.in $DISTDIR/fserve.bat
install -m 0755 finsert.jar.in $DISTDIR/finsert.bat
install -m 0755 frequest.jar.in $DISTDIR/frequest.bat

# make zip file
echo "Creating zip archive $ZIPFILE"
(cd $TMP; zip -Dr - $RELATIVE_DISTDIR) > $ZIPFILE

# cleanup
echo "Removing temporary directory $DISTDIR"
rm -rf $DISTDIR
echo "done"
