#!/bin/bash
#
# install.sh - install Freenet
#


# display banner intro
cat <<-EOF

Freenet installation
	
You are about to install a beta quality release of Freenet.  This means
that we *know* there are many bugs in it but we need you to test Freenet to
let us know what they are.  Freenet is being updated daily so it is
recommended that you regularly download and install a new snapshot.  You
should not be surprised if Freenet doesn't compile, or doesn't run -- but
if this happens please email freenet-dev@lists.sourceforge.net to let the
development team know that there is a problem.

WARNING: This release of Freenet is a proof-of-concept to test scalability
and routing.  It does not (yet) provide any meaningful security!  Please do
not insert or request any material which would place you at risk if your
anonymity were compromised.  If you require strong anonymity, please wait
for the next release.  However, if you want to help us test and develop the
system, please carry on!  

This script will install the Freenet server and Java client.
EOF


# check that Freenet was built already
echo -n "checking for compiled Node.class... "
if test ! -e ../node/Node.class; then
    echo "no"
    echo "Please run the ./build.sh script first."
    echo ""
    exit
fi
echo "yes"


# check for java runtime
echo -n "checking for java runtime... "
javaloc=`which java`
if test $? = 0; then
    echo "$javaloc"
else
    echo "no"
    echo "Please download a Java Development Kit (try http://java.sun.com)"
    echo ""
    exit
fi


# check java version
echo -n "checking java version is 1.1 or 1.2... "
version=`$javaloc -version 2>&1 | head -1`
echo "$version"
if echo "$version" | grep -Ev '1.1|1.2' > /dev/null; then
    echo ""
    echo "Freenet requires java compatible with Sun's 1.1 or 1.2 API"
    echo -n "I can't tell if yours qualifies.  Continue anyway [y/N]? "
    read bsh
    if test "$bsh" != "y"; then
	echo "Please download Java 1.2 (try http://java.sun.com)"
	exit
    fi
fi
echo ""


# are we running as root?
if test "$(whoami)" != "root"; then
    root="no"
else
    root="yes"
fi


# get install path
if test "$root" = "yes"; then
    PREFIX="/usr/local"
else
    PREFIX="$HOME"
    echo "Make sure to specify a path where you have write permission."
fi
echo -n "Where do you want to install Freenet [$PREFIX/freenet]? "
read foo
if test -n "$foo"; then
    INSTALLDIR="$foo"
else
    INSTALLDIR="$PREFIX/freenet"
fi


# get local executable path and check permissions
echo -n "Should I make symbolic links to the Freenet scripts in $PREFIX/bin [Y/n]? "
read foo
if test "$foo" != "n"; then
    BINDIR="$PREFIX/bin"
    if test ! -w "$BINDIR"; then
	echo "$BINDIR is not writable!"
	exit
    fi
else
    BINDIR=""
    echo "ok, $INSTALLDIR/bin must be in the PATH environment variable"
    echo "of all users who will run the Freenet server"
fi


# Copy class files
echo "Creating $INSTALLDIR and copying files..."
install -d -m 0755 $INSTALLDIR
install -m 0644 sample.freenetrc $INSTALLDIR/.freenetrc
cd ..
echo "Creating $INSTALLDIR/Freenet"
install -d -m 0755 $INSTALLDIR/Freenet
install -m 0644 *.class $INSTALLDIR/Freenet
for dir in message support crypt client node; do
    echo "Creating $INSTALLDIR/Freenet/$dir"
    install -d -m 0755 $INSTALLDIR/Freenet/$dir
    install -m 0644 $dir/*.class $INSTALLDIR/Freenet/$dir
done


# create server invocation script
cd scripts
echo "Creating $INSTALLDIR/bin"
install -d -m 0755 $INSTALLDIR/bin
echo "Creating $INSTALLDIR/bin/freenet_server executable"
sed "s#INSTALLDIR#$INSTALLDIR#g" freenet_server.in > freenet_server
install -m 0755 freenet_server $INSTALLDIR/bin/freenet_server
rm -f freenet_server


# create client invocation script (x2)
echo "Creating $INSTALLDIR/bin/freenet_insert executable"
sed "s#INSTALLDIR#$INSTALLDIR#g" freenet_insert.in > freenet_insert
echo "Creating $INSTALLDIR/bin/freenet_request executable"
install -m 0755 freenet_insert $INSTALLDIR/bin/freenet_insert
ln -f $INSTALLDIR/bin/freenet_insert $INSTALLDIR/bin/freenet_request
rm -f freenet_insert


# make symlinks
if test -n "$BINDIR"; then
    echo "Making symlinks in $BINDIR"
    ln -sf $INSTALLDIR/bin/freenet_server $BINDIR/freenet_server
    ln -sf $INSTALLDIR/bin/freenet_insert $BINDIR/freenet_insert
    ln -sf $INSTALLDIR/bin/freenet_request $BINDIR/freenet_request
fi


# done
cat <<-EOF

	Installation done.  Type 'freenet_server [options]' to start the server,
	or 'freenet_server -help' for usage information.  Check freenet.log in 
	your current directory for any error messages once the server is running.

	Type 'freenet_insert' or 'freenet_request' to get usage information for
	the client.

EOF

